//----------------------------------------------------------------------------------------------------------------------
// cloptions	PROCESS COMMAND LINE			created on 	27 Jun 2013						Copyright Geoff Kaniuk 2013
//----------------------------------------------------------------------------------------------------------------------
//  This file is part of Tio.
//
//  Tio is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  Tio is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Tio.  If not, see <http://www.gnu.org/licenses/>.
//----------------------------------------------------------------------------------------------------------------------
// A copy of the GNU General Public License is released with this software in the file COPYING
//----------------------------------------------------------------------------------------------------------------------
  
#include "cloptions.h"

//----------------------------------------------------------------------------------------------------------------------
// CLASS Commandline
//
namespace Tio
{
	enum																	// Error return values for get_option
	{
		ecl_missing=-1, ecl_multiple=-2, ecl_unknown=-3
	};

	Commandline::Commandline(const int pnopt, const char** opnames)
	:noshortoptions("\0"), options(0), nopt(pnopt)
	{
		if(nopt>0 and opnames)
		{
			options = new option[nopt+1];

			for (int iopt = 0; iopt < nopt; ++iopt)
				options[iopt] = (option){ opnames[iopt], no_argument, 0, iopt };

			options[nopt] = (option){ 0,0,0,0 };
		}
	}

	Commandline::~Commandline()
	{
		delete[] options;
	}

	int Commandline::get_option(const int argc, char *const *argv)
	{
		const int npar = argc - 2;	if(npar<0) return ecl_missing;			// Exactly one long option

		int nlongopt=0, iopt=0, parbegin=0;

		while(true)
		{
			const int longopt = getopt_long(argc, argv, noshortoptions, options, &parbegin);

			if(longopt>=0)
			{
				iopt = longopt;	++nlongopt;
			}
			else
				break;
		}
		if(nlongopt != 1) return ecl_multiple;								// Expecting one long option

		if(0<=iopt && iopt <nopt)
		{
			params.resize(npar);

			for (int p = 0; p < npar; ++p)
	            params[p] = argv[optind++];

			return iopt;
		}
		else
			return ecl_unknown;
	}

	const char* Commandline::get_parameter(const size_t par)
	{
		if(0<=par && par<params.size())
			return params[par];
		else
			return 0;
	}
}
// END cloptions
//----------------------------------------------------------------------------------------------------------------------
