//----------------------------------------------------------------------------------------------------------------------
// error	ERROR REPORTING 	created on 3 Mar 2013										Copyright Geoff Kaniuk 2013
//----------------------------------------------------------------------------------------------------------------------
//
//  This file is part of toursim.
//
//  toursim is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  toursim is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with toursim.  If not, see <http://www.gnu.org/licenses/>.
//----------------------------------------------------------------------------------------------------------------------
// A copy of the GNU General Public License is released with this software in the file COPYING
//----------------------------------------------------------------------------------------------------------------------

#include <iostream>
#include <sstream>

#include "error.h"

//----------------------------------------------------------------------------------------------------------------------
// DEBUG AIDS
//
namespace Tio
{
	int ipause=0;

	extern void pause(void)
	{
		++ipause;
	}
}

//----------------------------------------------------------------------------------------------------------------------
// ERROR REPORTING
//
namespace Tio
{
	using std::cout;
	using std::clog;
	using std::endl;

	Error::Error(const char* mprefixp, const int nmessagep,  const char** messagesp)
	:mprefix(mprefixp),
	 nmessage(nmessagep<0 || !messagesp ? 0 : nmessagep),
	 messages(nmessagep>0 ? messagesp : 0)
	{
		nerror = 0;
		errno = 0;
	}

	const char* Error::get_error_text(void)const
	{
		if(errno<0 or errno>=nmessage) return "Error::errno - corrupted";

		if(messages==0) return "Error::messages - null";

		return messages[errno];
	}

	void Error::message(const int errnop)
	{
		errno = (0<=errnop && errnop<nmessage) ? errnop : nmessage;

		if(clog.good())
			clog<<"ERR:"<<mprefix<<":"<<(errno<nmessage ? messages[errno] : "unknown");
	}

	void Error::warning(const int errnop)
	{
		if(0<=errnop && errnop<nmessage)
			clog<<"WARN:"<<mprefix<<":"<<messages[errnop];
		else
			clog<<"WARN:"<<mprefix<<":unknown";
	}

	bool Error::logerr(void)
	{
		if(nerror> max_error_log+1) return false;

		if(clog.fail())
		{
			cout<<"CANNOT LOG ERROR"<<endl;		return false;
		}

		switch(nerror)
		{
			case max_error_log+0:	cout<<"TOO MANY LOGGED ERRORS"<<endl; break;
			case max_error_log+1: 	clog<<"TOO MANY LOGGED ERRORS"<<endl; break;

			default:													  break;
		}

		return nerror++ < max_error_log;
	}

	bool Error::fail(const int errnop)
	{
		if(logerr())
		{
			message(errnop);	clog<<endl;
		}
		return false;
	}

	bool Error::fail(const int errnop, const int ival)
	{
		if(logerr())
		{
			message(errnop);	clog<<":"<<ival<<endl;
		}
		return false;
	}

	bool Error::fail(const int errnop, const string& text)
	{
		if(logerr())
		{
			message(errnop);	clog<<":"<<text<<endl;
		}
		return false;
	}

	bool Error::fail(const int errnop, const char* text)
	{
		if(logerr())
		{
			message(errnop);	clog<<":"<<text<<endl;
		}
		return false;
	}

	void Error::warn(const int errnop)
	{
		if(logerr()){ message(errnop);	clog<<endl; }
	}

	void Error::warn(const int errnop, const int ival)
	{
		if(logerr()){ message(errnop);	clog<<":"<<ival<<endl; }
	}

	void Error::warn(const int errnop, const string& text)
	{
		if(logerr()){ message(errnop);	clog<<":"<<text<<endl; }
	}

	void Error::warn(const int errnop, const char* text)
	{
		if(logerr()){ message(errnop);	clog<<":"<<text<<endl; }
	}

}

// END error
//----------------------------------------------------------------------------------------------------------------------
