//----------------------------------------------------------------------------------------------------------------------
// error	Error reporting		created on 3 Mar 2013 										Copyright Geoff Kaniuk 2013
//----------------------------------------------------------------------------------------------------------------------
//
//  This file is part of toursim.
//
//  toursim is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  toursim is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with toursim.  If not, see <http://www.gnu.org/licenses/>.
//----------------------------------------------------------------------------------------------------------------------
// A copy of the GNU General Public License is released with this software in the file COPYING
//----------------------------------------------------------------------------------------------------------------------

#include <string>

#ifndef ERROR_H_
#define ERROR_H_

namespace Tio
{
	using std::string;

	class Error
	{
		private:
			const char*  mprefix;	// Module prefix code

			const int    nmessage;	// Number of messages
			const char** messages;	// Assume size is nmessage

			int errno;				// Error number
			int nerror;				// Number of errors logged

		private:
			bool logerr(void);														// Limit number of logged messages

			void message(const int errnop);											// Internal workhorse - no endl
			void warning(const int errnop);

		public:
			enum { max_error_log = 20 };

			Error
			(
				const char* mprefixp,
				const int nmessagep, const char** messagesp
			);

			int get_errno(void)const { return errno; }

			const char* get_error_text(void)const;

			bool fail(const int errnop);
			bool fail(const int errnop, const int ival);
			bool fail(const int errnop, const string& text);
			bool fail(const int errnop, const char* text);

			void warn(const int errnop);
			void warn(const int errnop, const int ival);
			void warn(const int errnop, const string& text);
			void warn(const int errnop, const char* text);
	};

	extern int ipause;
	extern void pause(void);
}
#endif 
// END error
//----------------------------------------------------------------------------------------------------------------------
